@@ -67,15 +67,11 @@ static TerminalFileInfo
 get_terminal_file_info (CajaFileInfo *file_info)
 {
 	TerminalFileInfo  ret;
-	char             *uri_scheme, *p;
+	char             *uri;
+	char             *uri_scheme;
 
-	g_assert (file_info);
-
-	uri_scheme = caja_file_info_get_activation_uri (file_info);
-	p = strchr (uri_scheme, ':');
-	if (p) {
-		*p = 0;
-	}
+	uri = caja_file_info_get_activation_uri (file_info);
+	uri_scheme = g_uri_parse_scheme (uri);
 
 	if (strcmp (uri_scheme, "file") == 0) {
 		ret = FILE_INFO_LOCAL;
@@ -89,6 +85,7 @@ get_terminal_file_info (CajaFileInfo *file_info)
 	}
 
 	g_free (uri_scheme);
+	g_free (uri);
 
 	return ret;
 }
@@ -135,40 +132,38 @@ static inline gboolean
 desktop_opens_home_dir (void)
 {
 	gboolean result;
-        GSettings* settings;
+	GSettings* settings;
 
-        settings = g_settings_new (COT_SCHEMA);
+	settings = g_settings_new (COT_SCHEMA);
 	result = g_settings_get_boolean (settings, COT_DESKTOP_KEY);
-        g_object_unref (settings);
-        return result;
+	g_object_unref (settings);
+	return result;
 }
 
 static inline gboolean
 desktop_is_home_dir (void)
 {
 	gboolean result;
-        GSettings* settings;
+	GSettings* settings;
 
-        settings = g_settings_new (CAJA_SCHEMA);
+	settings = g_settings_new (CAJA_SCHEMA);
 	result = g_settings_get_boolean (settings, CAJA_DESKTOP_KEY);
-        g_object_unref (settings);
-        return result;
+	g_object_unref (settings);
+	return result;
 }
 
 static inline gchar*
 default_terminal_application (void)
 {
 	gchar *result;
-        GSettings* settings;
+	GSettings* settings;
 
-        settings = g_settings_new (TERM_SCHEMA);
+	settings = g_settings_new (TERM_SCHEMA);
 	result = g_settings_get_string (settings, TERM_EXEC_KEY);
-        g_object_unref (settings);
-        return result;
+	g_object_unref (settings);
+	return result;
 }
 
-#define get_desktop_dir() g_strdup (g_get_user_special_dir (G_USER_DIRECTORY_DESKTOP))
-
 static void
 parse_sftp_uri (GFile *file, char **host, guint *port, char **user,
 		char **path)
@@ -226,6 +221,9 @@ append_sftp_info (char **terminal_exec,
 	char *host_name, *path, *user_name;
 	char *user_host, *cmd, *quoted_cmd;
 	char *host_port_switch;
+	char *quoted_path;
+	char *remote_cmd;
+	char *quoted_remote_cmd;
 	guint host_port;
 
 	g_assert (terminal_exec != NULL);
@@ -252,7 +250,11 @@ append_sftp_info (char **terminal_exec,
 		user_host = g_strdup (host_name);
 	}
 
-	cmd = g_strdup_printf ("ssh %s %s -t \"cd \'%s\' && $SHELL -l\"", user_host, host_port_switch, path);
+	quoted_path = g_shell_quote (path);
+	remote_cmd = g_strdup_printf ("cd %s && $SHELL -l", quoted_path);
+	quoted_remote_cmd = g_shell_quote (remote_cmd);
+
+	cmd = g_strdup_printf ("ssh %s %s -t %s", user_host, host_port_switch, quoted_remote_cmd);
 	quoted_cmd = g_shell_quote (cmd);
 	g_free (cmd);
 
@@ -264,7 +266,10 @@ append_sftp_info (char **terminal_exec,
 	g_free (user_name);
 	g_free (host_port_switch);
 	g_free (path);
+	g_free (quoted_path);
 
+	g_free (remote_cmd);
+	g_free (quoted_remote_cmd);
 	g_free (quoted_cmd);
 	g_free (user_host);
 	g_object_unref (vfs_uri);
@@ -274,7 +279,8 @@ static void
 open_terminal_callback (CajaMenuItem *item,
 			CajaFileInfo *file_info)
 {
-	gchar *display_str;
+	GdkDisplay   *display;
+	const gchar *display_str;
 	const gchar *old_display_str;
 	gchar *uri;
 	gchar **argv, *terminal_exec;
@@ -305,7 +311,7 @@ open_terminal_callback (CajaMenuItem *item,
 			if (desktop_is_home_dir () || desktop_opens_home_dir ()) {
 				working_directory = g_strdup (g_get_home_dir ());
 			} else {
-				working_directory = get_desktop_dir ();
+				working_directory = g_strdup (g_get_user_special_dir (G_USER_DIRECTORY_DESKTOP));
 			}
 			break;
 
@@ -331,8 +337,9 @@ open_terminal_callback (CajaMenuItem *item,
 	old_display_str = g_getenv ("DISPLAY");
 
 	screen = g_object_get_data (G_OBJECT (item), "CajaOpenTerminal::screen");
+	display = gdk_screen_get_display (screen);
 	if (screen != NULL) {
-		display_str = gdk_screen_make_display_name (screen);
+		display_str = gdk_display_get_name (display);
 		g_setenv ("DISPLAY", display_str, TRUE);
 	}
 
@@ -408,13 +415,13 @@ open_terminal_callback (CajaMenuItem *item,
 	g_strfreev (argv);
 	g_free (terminal_exec);
 	g_free (working_directory);
-	g_free (display_str);
 }
 
 static CajaMenuItem *
-open_terminal_menu_item_new (TerminalFileInfo  terminal_file_info,
-			     GdkScreen        *screen,
-			     gboolean          is_file_item)
+open_terminal_menu_item_new (CajaFileInfo	  *file_info,
+                             TerminalFileInfo  terminal_file_info,
+                             GdkScreen        *screen,
+                             gboolean          is_file_item)
 {
 	CajaMenuItem *ret;
 	const char *name;
@@ -448,17 +455,25 @@ open_terminal_menu_item_new (TerminalFileInfo  terminal_file_info,
 
 	ret = caja_menu_item_new ("CajaOpenTerminal::open_terminal",
 				      name, tooltip, "terminal");
+
 	g_object_set_data (G_OBJECT (ret),
 			   "CajaOpenTerminal::screen",
 			   screen);
 
+	g_object_set_data_full (G_OBJECT (ret), "file-info",
+				g_object_ref (file_info),
+				(GDestroyNotify) g_object_unref);
+	g_signal_connect (ret, "activate",
+			  G_CALLBACK (open_terminal_callback),
+			  file_info);
+
 	return ret;
 }
 
 static GList *
 caja_open_terminal_get_background_items (CajaMenuProvider *provider,
-					     GtkWidget		  *window,
-					     CajaFileInfo	  *file_info)
+                                         GtkWidget        *window,
+                                         CajaFileInfo     *file_info)
 {
 	CajaMenuItem *item;
 	TerminalFileInfo  terminal_file_info;
@@ -468,14 +483,7 @@ caja_open_terminal_get_background_items (CajaMenuProvider *provider,
 		case FILE_INFO_LOCAL:
 		case FILE_INFO_DESKTOP:
 		case FILE_INFO_SFTP:
-			item = open_terminal_menu_item_new (terminal_file_info, gtk_widget_get_screen (window), FALSE);
-			g_object_set_data_full (G_OBJECT (item), "file-info",
-						g_object_ref (file_info),
-						(GDestroyNotify) g_object_unref);
-			g_signal_connect (item, "activate",
-					  G_CALLBACK (open_terminal_callback),
-					  file_info);
-
+			item = open_terminal_menu_item_new (file_info, terminal_file_info, gtk_widget_get_screen (window), FALSE);
 			return g_list_append (NULL, item);
 
 		case FILE_INFO_OTHER:
@@ -488,8 +496,8 @@ caja_open_terminal_get_background_items (CajaMenuProvider *provider,
 
 GList *
 caja_open_terminal_get_file_items (CajaMenuProvider *provider,
-				       GtkWidget            *window,
-				       GList                *files)
+                                   GtkWidget        *window,
+                                   GList            *files)
 {
 	CajaMenuItem *item;
 	TerminalFileInfo  terminal_file_info;
@@ -505,14 +513,7 @@ caja_open_terminal_get_file_items (CajaMenuProvider *provider,
 	switch (terminal_file_info) {
 		case FILE_INFO_LOCAL:
 		case FILE_INFO_SFTP:
-			item = open_terminal_menu_item_new (terminal_file_info, gtk_widget_get_screen (window), TRUE);
-			g_object_set_data_full (G_OBJECT (item), "file-info",
-						g_object_ref (files->data),
-						(GDestroyNotify) g_object_unref);
-			g_signal_connect (item, "activate",
-					  G_CALLBACK (open_terminal_callback),
-					  files->data);
-
+			item = open_terminal_menu_item_new (files->data, terminal_file_info, gtk_widget_get_screen (window), TRUE);
 			return g_list_append (NULL, item);
 
 		case FILE_INFO_DESKTOP:
